(:~
 : This module creates the single edition view. According to the GET parameter
 : "edition" this module gets from edition.xqm all common text lines and the ones
 : that fit a tei:rdg are selected.
 :
 : The whole module is called by xml-utils.xqm.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 :)

xquery version "3.1";

module namespace single="http://bdn-edition.de/ns/single";

declare namespace bdn="http://www.bdn-edition.de/ns/1.0";
declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace xhtml="http://www.w3.org/1999/xhtml";
declare namespace html5="http://www.w3.org/1999/xhtml";

import module namespace commons="http://bdn-edition.de/ns/commons" at "commons.xqm";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace functx = "http://www.functx.com";
import module namespace id-file="https://bdn-edition.de/ns/id-file" at "id-file.xqm";

declare variable $single:id-map :=
  map:merge
  (
    for $id in doc($config:aux || "id-file.xml")//bdn:id
        return map:entry($id/string(), $id/parent::bdn:file/@uri/string())
  );
(: Wozu wird die Variable $single:id-map genutzt?! :)

(:~
 : This function calls the witness-element according to the current edition.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 :)

declare function single:witness( $node, $mode ){
    $node/root()//tei:witness[@xml:id = $mode]
};

(:~
 : This function calls the rend-attribute of the witness-element according to the current edition.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 :)

declare function single:wit-rend( $node, $mode ){
    data(single:witness($node, $mode)/@rend)
};

(:~
 : This function creates a div with a certain class and calls the functions single:apply-corresp-node and single:transformn.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $class: Defining the class
 :)

declare function single:divclass
    ( $node, $mode, $file, $class as xs:string ){
    element html5:div {
        attribute class { $class },
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
        else
            single:transform($node/node(), $mode, $file)
    }
};

(:~
 : This function creates a q with a certain class and calls the functions single:apply-corresp-node and single:transformn.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $class: Defining the class
 :)

declare function single:qclass
    ( $node, $mode, $file, $class as xs:string ){
    element html5:q {
        attribute class { $class },
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
        else
            single:transform($node/node(), $mode, $file)
    }
};

(:~
 : This function creates a span with a certain class and calls the functions single:apply-corresp-node and single:transformn.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $class: Defining the class
 :)

declare function single:spanclass
    ( $node, $mode, $file, $class as xs:string ){
    element html5:span {
        attribute class { $class },
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
        else
            single:transform($node/node(), $mode, $file)
    }
};

(:~
 : This function creates html5:h1, html5:h2, html5:h3, html5:h4, or html5:h5. It uses functions single:apply-corresp-node and single:transformn.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $class: Defining the class
 :)
(::)
(:declare function single:head:)
(:    ( $node, $mode, $file )  {:)
(:    element html5:h1:)
(:    {:)
(:      if ($node[@type = "sub"]) then:)
(:        attribute class {"sub-head"}:)
(:      else:)
(:        attribute class {"list-main-head"},:)
(:      if ($node/@copyOf) then:)
(:        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false()):)
(:      else:)
(:        single:transform($node/node(), $mode, $file):)
(:    }:)
(:};:)

(:~
 : This function calls the element with the full-link.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 :)

declare function single:linkelement( $node ){
    $node/root()//*[@xml:id = data(replace($node/attribute()[name() = ('corresp', 'ref', 'target', 'resp')], '#', ''))][1]
};

(:~
 : This function calls the ref-attribute of the element with the full-link. It calls the function single:linkelement.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

declare function single:fullink( $node ){
    single:linkelement($node)/attribute()[name() = ('corresp', 'ref', 'target', 'resp')]
};

(:~
 : This function replaces the internal link with the full link and creates an icon that works as a link. It calls the function single:fulllink.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $abbrev: placeholder in the TEI-XML
 : @param $normalization: Base-URL of the actual linktarget
 : @param $suffix: Adding a suffix to the link
 : @param $icon: Defines the kind of icon
 :)

declare function single:fulllinkicon
    ( $node, $abbrev as xs:string, $normalization as xs:string, $suffix as xs:string, $icon as xs:string ){
    element html5:a {
        attribute class {"text-link"},
        attribute href {replace(data(single:fullink($node)), $abbrev, $normalization) || $suffix},
        element html5:i {
            attribute class { $icon }
        }
    }
};

(:~
 : This function creates an icon that works as a link.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $abbrev: placeholder in the TEI-XML
 : @param $normalization: Base-URL of the actual linktarget
 : @param $suffix: Adding a suffix to the link
 : @param $icon: Defines the kind of icon
 :)

declare function single:linkicon
    ( $node, $abbrev as xs:string, $normalization as xs:string, $suffix as xs:string, $icon as xs:string ){
    if ($abbrev = "zotero:") then
        single:linkzoteroPopover($node, $icon)
    else (
        element html5:a {
            attribute class {"text-link"},
            attribute href {replace($node/attribute()[name() = ('corresp', 'ref', 'target')], $abbrev, $normalization) || $suffix},
            element html5:i {
                attribute class { $icon }
            }
        }
        )

};

(:~
 : Creates a zotero link popover. It is just a wrapper around the
 : generic function createPopover and sets the correct id by
 : replacing zotero: with just the id. It sets the title
 : to Literaturnachweis and the default content is Laden ...
 : until Javascript replaces the content
 :
 : @author Johannes Biermann
 : @version 1.0
 : 
 : @param $icon: Defines the kind of icon
 : @param $node: The current node
:)
declare function single:linkzoteroPopover($node, $icon as xs:string) {
    single:createPopover(
        $icon,
        'kmg-bib',
        'Literaturnachweis',
        'Laden ...',
        replace($node/attribute()[name() = ('corresp', 'ref', 'target')], "zotero:", "")
        )
};

(:~
 : This function creates a popover https://getbootstrap.com/docs/3.3/javascript/#popovers
 :  with a font-awesome icon
 : A popover is a thing where you can click to see some more information, another 
 : click somewhere else closes ist.
 : 
 : @author Johannes Biermann
 : @version 1.0
 : @param $icon: Defines the kind of icon
 : @param $class: the html class for this element type. Usually you use the class to bind
 : javascript actions for a popover. See kmg.js for an example
 : you can leave this empty for static popovers
 : @param $title: the title for the popover box. The box has two rows: title and content
 : @param $content: the content of the popover. Via Javascript you can replace this later
 : or leave it static
 : @param $id: an ID which will be set to data-id and can be accessed via Javascript
 : you can leave this empty for static popovers
 :)
declare function single:createPopover($icon as xs:string, $class as xs:string, $title as xs:string, $content as xs:string, $id as xs:string) {
    element html5:a {
        attribute class {$class},
        attribute data-container {"body"},
        attribute data-content {$content},
        attribute data-html {"data-html"},
        attribute data-placement {"bottom"},
        attribute data-title {$title},
        attribute data-toggle {"popover"},
        attribute data-trigger {"focus"},
        attribute data-id {$id},
        attribute role {"button"},
        attribute tabindex {"-1"},
        attribute data-original-title {""},
        attribute title {""},
        element html5:i {
            attribute class { $icon }
        }        
    }
};

(:~
 : This function creates a span around the text that was tagged as some kind of link and calls the functions single:apply-corresp-node, single:transform, and single:fulllinkicon or single:linkicon.
 :
 : @author Max Zeterberg
 : @version 1.0
 : @param $node: The current node
 : @param $mode: The current edition
 : @param $class: Defining a certain class for the link
 :)

declare function single:spanicon
    ( $node, $mode, $file, $class as xs:string, $abbrev as xs:string, $normalization as xs:string, $suffix as xs:string, $icon as xs:string ){
    (element html5:span {
        attribute class { $class },
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
        else
            single:transform($node/node(), $mode, $file),
        string(" "),
        if (starts-with($node/attribute()[name() = ('corresp', 'ref', 'target')], "#" )) then
            single:fulllinkicon($node, $abbrev, $normalization, $suffix, $icon)
        else
            single:linkicon($node, $abbrev, $normalization, $suffix, $icon),
        if (commons:starts-next-text-with-punctuation($node, $mode)) then
            ()
        else
            text{" "}
        }
    )
};

declare function single:transform($nodes as node()*, $mode as xs:string*,
$file as xs:string) as node()*
{
  for $node in $nodes
  return
    (
      if ($node/@xml:id) then
        commons:make-text-anchor(string($node/@xml:id)
        (: Hier werden alle Elemente mit xml:id mit html-Ankerpunkten versehen :)
    )
    else
      (),
    typeswitch($node)
    case text() return
      let $text := text{replace($node, "@", " ")}
      return
(:          let $app := $node/following-sibling::*[1][self::tei:app]:)
(:          let $next-text := $app/following-sibling::node()[1][self::text()]:)
(:          let $next-text-first-char := substring($next-text, 1, 1):)
(:          return:)
(:              (: example: Griesbach 250bg, #a "einen neuen Zuwachs, wenn man":):)
(:              if ($app/tei:rdg[matches(@wit, $mode) and @type = "om"]:)
(:              and matches($next-text-first-char, "[,\.\?!\)]")) then:)
(:                  text{functx:right-trim($text)}:)
(:              else:)
                  $text

    case element(tei:app) return
      (if ($node/child::*[matches(@wit, $mode)]) then
        single:transform($node/child::*[matches(@wit, $mode)], $mode, $file)
      else
        single:transform($node/child::tei:lem, $mode, $file),
      if (single:is-next-text-in-app($node)
      and single:is-next-app-omitted($node, $mode)) then
        ()
      else
        commons:make-whitespace-after($node))

    case comment() return
      ()

    case element(tei:div) return
      switch($node/@type)
      case ""
        return
        element html5:div
        {
        if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
        else
            single:transform($node/node(), $mode, $file)
        }
      default return
        element html5:div
        {
          if ($node/@copyOf) then
            single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
          else
            single:transform($node/node(), $mode, $file)
        }

    case element(tei:foreign) return
      if ($node[@xml:lang = "el"]) then
          single:spanclass( $node, $mode, $file, "hi-greek" )
      else
          single:spanclass( $node, $mode, $file, "foreign" )

    case element(tei:front) return
      element html5:div
      {
        attribute class {"preface"},
        single:transform($node/node(), $mode, $file)
      }

    case element(tei:head) return
        if ($node/parent::tei:figure) then
            ()
        else if ($node/parent::tei:table) then
            element html5:caption
            {
                if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                single:transform($node/node(), $mode, $file)
            }
        else
            (if ($node/parent::tei:div[@type = "part"]) then
                element html5:h2
                {
                  if ($node[@type = "sub"]) then
                    attribute class {"sub-head"}
                  else
                    attribute class {"list-main-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            else if ($node/parent::tei:div[@type = "chapter"]) then
                element html5:h3
                {
                  if ($node[@type = "sub"]) then
                    attribute class {"sub-head"}
                  else
                    attribute class {"list-main-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            else if ($node/parent::tei:div[@type = "section"]) then
                element html5:h4
                {
                  if ($node[@type = "sub"]) then
                    attribute class {"sub-head"}
                  else
                    attribute class {"list-main-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            else
                element html5:h5
                {
                  if ($node[@type = "sub"]) then
                    attribute class {"sub-head"}
                  else
                    attribute class {"list-main-head"},
                  if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                  else
                    single:transform($node/node(), $mode, $file)
                }
            )

    case element(tei:hi) return
      if ($node/@rend) then
          if ($node[@rend = "spaced"] and matches(substring($node/following::text()[1], 1, 1), "[,!\?\.:;]")) then
              single:spanclass( $node, $mode, $file, "spaced-out spaced-out-no-right-padding" )
          else
              single:spanclass( $node, $mode, $file, $node/@rend )
      else
        (if (contains(single:wit-rend($node, $mode), "body_type")) then
            single:spanclass( $node, $mode, $file, "body_type" )
        else if (contains(single:wit-rend($node, $mode), "bold")) then
            single:spanclass( $node, $mode, $file, "bold" )
        else if (contains(single:wit-rend($node, $mode), "italics")) then
            single:spanclass( $node, $mode, $file, "italics" )
        else if (contains(single:wit-rend($node, $mode), "small_caps")) then
            single:spanclass( $node, $mode, $file, "small_caps" )
        else if (contains(single:wit-rend($node, $mode), "spaced") and matches(substring($node/following::text()[1], 1, 1), "[,!\?\.:;]")) then
            single:spanclass( $node, $mode, $file, "spaced spaced-out-no-right-padding" )
        else if (contains(single:wit-rend($node, $mode), "spaced")) then
            single:spanclass( $node, $mode, $file, "spaced" )
        else if (contains(single:wit-rend($node, $mode), "under")) then
            single:spanclass( $node, $mode, $file, "under" )
        else if (contains(single:wit-rend($node, $mode), "versal")) then
            single:spanclass( $node, $mode, $file, "versal" )
        else
            single:spanclass( $node, $mode, $file, "hi" )
        )

    case element(tei:label) return
      if ($node[@type = "note_anchor"]) then
          (if ($node/parent::*[@anchored = "false"]) then
              element html5:span
                {
                  attribute class {"anchored_false"},
                  single:transform($node/node(), $mode, $file),
                  text {" "}
                }
            else
                element html5:span
                {
                  attribute class {"note_anchor"},
                  single:transform($node/node(), $mode, $file),
                  text {" "}
                }
          )
      else if ($node[@type = "list"]) then
          ()
      else if ($node[@type = "head"]) then
          single:spanclass( $node, $mode, $file, "headlabel" ) 
      else
          single:spanclass( $node, $mode, $file, "label-wrapper" )

    case element(tei:list) return
        if ($node/@type = ("unspecified", "gloss", "index")) then
            element html5:ul
              {
                (if ($node/parent::tei:item) then
                  attribute class {"nested-list unlabeled_list"}
                else if ($node/ancestor::tei:rdg) then
                  attribute class {"rdg-list unlabeled_list"}
                else
                  attribute class {"text-list unlabeled_list"}),
                single:transform($node/node(), $mode, $file)
              }
        else if ($node/@type = "abbildungstext") then
            ()
        else
          element html5:ul
          {
            (if ($node/parent::tei:item) then
              attribute class {"nested-list labeled_list"}
            else if ($node/ancestor::tei:rdg) then
              attribute class {"rdg-list labeled_list"}
            else
              attribute class {"text-list labeled_list"}),
            single:transform($node/node(), $mode, $file)
          }

    case element(tei:l) return
        if ($node/parent::tei:lg/parent::tei:quote) then
            (if (not($node/preceding-sibling::tei:l)) then
                (if ($node/parent::tei:lg/parent::*[contains(@rend, "double")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        single:spanclass( $node, $mode, $file, "line quote double-qm-low-l" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        single:spanclass( $node, $mode, $file, "line quote double-qm-high-l" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        single:spanclass( $node, $mode, $file, "line quote double-g-out-l")
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        single:spanclass( $node, $mode, $file, "line quote double-g-in-l")
                    else
                        (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                            single:spanclass( $node, $mode, $file, "line quote double-qm-low-l")
                        else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                            single:spanclass( $node, $mode, $file, "line quote double-qm-high-l")
                        else if (contains(single:wit-rend($node, $mode), "g_out")) then
                            single:spanclass( $node, $mode, $file, "line quote double-g-out-l")
                        else if (contains(single:wit-rend($node, $mode), "g_in")) then
                            single:spanclass( $node, $mode, $file, "line quote double-g-in-l")
                        else
                            single:spanclass( $node, $mode, $file, "line quote double-l")
                        )
                    )
                else if ($node/parent::tei:lg/parent::*[contains(@rend, "single")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        single:spanclass( $node, $mode, $file, "line quote single-qm-low-l" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        single:spanclass( $node, $mode, $file, "line quote single-qm-high-l" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        single:spanclass( $node, $mode, $file, "line quote single-g-out-l")
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        single:spanclass( $node, $mode, $file, "line quote single-g-in-l")
                    else
                        (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                            single:spanclass( $node, $mode, $file, "line quote single-qm-low-l")
                        else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                            single:spanclass( $node, $mode, $file, "line quote single-qm-high-l")
                        else if (contains(single:wit-rend($node, $mode), "g_out")) then
                            single:spanclass( $node, $mode, $file, "line quote single-g-out-l")
                        else if (contains(single:wit-rend($node, $mode), "g_in")) then
                            single:spanclass( $node, $mode, $file, "line quote single-g-in-l")
                        else
                            single:spanclass( $node, $mode, $file, "line quote single-l")
                        )
                    )
                else
                    single:spanclass( $node, $mode, $file, "line quote single-l" )
                )
            else if (not($node/following-sibling::tei:l)) then
                (if ($node/parent::tei:lg/parent::*[contains(@rend, "double")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        single:spanclass( $node, $mode, $file, "line quote double-qm-low-r" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        single:spanclass( $node, $mode, $file, "line quote double-qm-high-r" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        single:spanclass( $node, $mode, $file, "line quote double-g-out-r")
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        single:spanclass( $node, $mode, $file, "line quote double-g-in-r")
                    else
                        (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                            single:spanclass( $node, $mode, $file, "line quote double-qm-low-r")
                        else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                            single:spanclass( $node, $mode, $file, "line quote double-qm-high-r")
                        else if (contains(single:wit-rend($node, $mode), "g_out")) then
                            single:spanclass( $node, $mode, $file, "line quote double-g-out-r")
                        else if (contains(single:wit-rend($node, $mode), "g_in")) then
                            single:spanclass( $node, $mode, $file, "line quote double-g-in-r")
                        else
                            single:spanclass( $node, $mode, $file, "line quote double-r")
                        )
                    )
                else if ($node/parent::tei:lg/parent::*[contains(@rend, "single")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        single:spanclass( $node, $mode, $file, "line quote single-qm-low-r" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        single:spanclass( $node, $mode, $file, "line quote single-qm-high-r" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        single:spanclass( $node, $mode, $file, "line quote single-g-out-r")
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        single:spanclass( $node, $mode, $file, "line quote single-g-in-r")
                    else
                        (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                            single:spanclass( $node, $mode, $file, "line quote single-qm-low-r")
                        else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                            single:spanclass( $node, $mode, $file, "line quote single-qm-high-r")
                        else if (contains(single:wit-rend($node, $mode), "g_out")) then
                            single:spanclass( $node, $mode, $file, "line quote single-g-out-r")
                        else if (contains(single:wit-rend($node, $mode), "g_in")) then
                            single:spanclass( $node, $mode, $file, "line quote single-g-in-r")
                        else
                            single:spanclass( $node, $mode, $file, "line quote single-r")
                        )
                    )
                else
                    single:spanclass( $node, $mode, $file, "line quote single-r" )
                )
            else
                single:spanclass( $node, $mode, $file, "line quote" )
            )
        else if ($node/parent::tei:lg/parent::tei:q) then
            (if (not($node/preceding-sibling::tei:l)) then
                (if ($node/parent::tei:lg/parent::*[contains(@rend, "double")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        single:spanclass( $node, $mode, $file, "line double-qm-low-l" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        single:spanclass( $node, $mode, $file, "line double-qm-high-l" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        single:spanclass( $node, $mode, $file, "line double-g-out-l")
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        single:spanclass( $node, $mode, $file, "line double-g-in-l")
                    else
                        (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                            single:spanclass( $node, $mode, $file, "line double-qm-low-l")
                        else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                            single:spanclass( $node, $mode, $file, "line double-qm-high-l")
                        else if (contains(single:wit-rend($node, $mode), "g_out")) then
                            single:spanclass( $node, $mode, $file, "line double-g-out-l")
                        else if (contains(single:wit-rend($node, $mode), "g_in")) then
                            single:spanclass( $node, $mode, $file, "line double-g-in-l")
                        else
                            single:spanclass( $node, $mode, $file, "line double-l")
                        )
                    )
                else if ($node/parent::tei:lg/parent::*[contains(@rend, "single")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        single:spanclass( $node, $mode, $file, "line single-qm-low-l" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        single:spanclass( $node, $mode, $file, "line single-qm-high-l" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        single:spanclass( $node, $mode, $file, "line single-g-out-l")
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        single:spanclass( $node, $mode, $file, "line single-g-in-l")
                    else
                        (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                            single:spanclass( $node, $mode, $file, "line single-qm-low-l")
                        else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                            single:spanclass( $node, $mode, $file, "line single-qm-high-l")
                        else if (contains(single:wit-rend($node, $mode), "g_out")) then
                            single:spanclass( $node, $mode, $file, "line single-g-out-l")
                        else if (contains(single:wit-rend($node, $mode), "g_in")) then
                            single:spanclass( $node, $mode, $file, "line single-g-in-l")
                        else
                            single:spanclass( $node, $mode, $file, "line single-l")
                        )
                    )
                else
                    single:spanclass( $node, $mode, $file, "line single-l" )
                )
            else if (not($node/following-sibling::tei:l)) then
                (if ($node/parent::tei:lg/parent::*[contains(@rend, "double")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        single:spanclass( $node, $mode, $file, "line double-qm-low-r" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        single:spanclass( $node, $mode, $file, "line double-qm-high-r" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        single:spanclass( $node, $mode, $file, "line double-g-out-r")
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        single:spanclass( $node, $mode, $file, "line double-g-in-r")
                    else
                        (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                            single:spanclass( $node, $mode, $file, "line double-qm-low-r")
                        else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                            single:spanclass( $node, $mode, $file, "line double-qm-high-r")
                        else if (contains(single:wit-rend($node, $mode), "g_out")) then
                            single:spanclass( $node, $mode, $file, "line double-g-out-r")
                        else if (contains(single:wit-rend($node, $mode), "g_in")) then
                            single:spanclass( $node, $mode, $file, "line double-g-in-r")
                        else
                            single:spanclass( $node, $mode, $file, "line double-r")
                        )
                    )
                else if ($node/parent::tei:lg/parent::*[contains(@rend, "single")]) then
                    (if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_low")]) then
                        single:spanclass( $node, $mode, $file, "line single-qm-low-r" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "qm_high")]) then
                        single:spanclass( $node, $mode, $file, "line single-qm-high-r" )
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_out")]) then
                        single:spanclass( $node, $mode, $file, "line single-g-out-r")
                    else if ($node/parent::tei:lg/parent::*[contains(@rend, "g_in")]) then
                        single:spanclass( $node, $mode, $file, "line single-g-in-r")
                    else
                        (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                            single:spanclass( $node, $mode, $file, "line single-qm-low-r")
                        else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                            single:spanclass( $node, $mode, $file, "line single-qm-high-r")
                        else if (contains(single:wit-rend($node, $mode), "g_out")) then
                            single:spanclass( $node, $mode, $file, "line single-g-out-r")
                        else if (contains(single:wit-rend($node, $mode), "g_in")) then
                            single:spanclass( $node, $mode, $file, "line single-g-in-r")
                        else
                            single:spanclass( $node, $mode, $file, "line single-r")
                        )
                    )
                else
                    single:spanclass( $node, $mode, $file, "line single-r" )
                )
            else
                single:spanclass( $node, $mode, $file, "line" )
            )
        else
            single:spanclass( $node, $mode, $file, "line" )

    case element(tei:lg) return
        element html5:div
          {
            attribute class {"linegroup"},
            single:transform($node/node(), $mode, $file)
          }

    case element(tei:rdg) return
        single:transform($node/node(), $mode, $file)

    (: Hier könnten noch die verschiedenen Anwendungsfälle spezifiziert werden :)
    case element(tei:seg) return
      if ($node/@copyOf) then
        single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
      else if ($node/@type = "margin") then
            element html5:span
            {
                attribute class {"margin"},
                text{"{"},
                single:transform($node/node(), $mode, $file),
                text{"}"}
            }
        else if ($node/@type = "list") then
            (if ($node/preceding-sibling::node()[not(self::text())][position() = 1]/local-name() = 'label') then
              element html5:li
              {
                if ($node/@xml:id) then
                  attribute id {$node/@xml:id}
                else
                  (),
                attribute class {"labeled_item"},
                <html5:span class="list-label">{data($node/preceding-sibling::tei:label[@type="list"][1])}</html5:span>,
                
                if ($node/@copyOf) then
                  single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                  single:transform($node/node(), $mode, $file)
              }
            else
               element html5:li
              {
                if ($node/@xml:id) then
                  attribute id {$node/@xml:id}
                else
                  (),
                attribute class {"unlabeled_item"},
                if ($node/@copyOf) then
                  single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                  single:transform($node/node(), $mode, $file)
              } 
            )
      else
        single:transform($node/node(), $mode, $file)

(:~
 : This transformation creates a representation of the tei:Header. It uses the function single:spanicon to create Link-Icons for tei:name (tei:author, tei:editor, tei:respStmt) and single:linkicon for tei:bibl (in tei:witness).
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

    case element(tei:teiHeader) return
        element html5:div
        {
            attribute class {"header kmg"},
            element html5:h1
            {
                single:transform($node/tei:fileDesc/tei:titleStmt/tei:title[@type="main"]/node(), $mode, $file)
            },
            if ($node/tei:fileDesc/tei:titleStmt/tei:title[@type="sub"]) then
                (element html5:h1
                {
                    attribute class {"sub-head"},
                    single:transform($node/tei:fileDesc/tei:titleStmt/tei:title[@type="sub"]/node(), $mode, $file)
                }
                )
            else
                (),
            element html5:p
            {
                attribute class {"headerperson"},
                "Autor: ",
                single:spanicon($node/tei:fileDesc/tei:titleStmt/tei:author/tei:name[1], $mode, $file, "persname", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-user")
            },
            if ($node/tei:fileDesc/tei:titleStmt/tei:respStmt) then
                (for $other in $node/tei:fileDesc/tei:titleStmt/tei:respStmt/tei:name[1]
                return
                    element html5:p
                {
                    attribute class {"headerperson"},
                    $other/preceding-sibling::tei:resp/text() || ": ",
                    single:spanicon($other, $mode, $file, "persname", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-user")
                })
            else
                (),
            element html5:h5
            {
                attribute class {"sub-head"},
                "Bearbeiter*innen:"
            },
            element html5:ul
            {
                for $editor in $node/tei:fileDesc/tei:titleStmt/tei:editor/tei:name[1]
                return
                    element html5:li
                    {
                        single:spanicon($editor, $mode, $file, "persname headerperson", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-user"),
                        " (" || string(replace($editor/parent::tei:editor/@role, " ", ", ")) || ")"
                    }
            },
            if (single:witness( $node/tei:fileDesc/tei:sourceDesc, $mode )/descendant::tei:witness[2]) then
                element html5:h5
                {
                    attribute class {"sub-head"},
                    "Dieser Version zugrundeliegende Publikationen:"
                }
            else
                element html5:h5
                {
                    attribute class {"sub-head"},
                    "Dieser Version zugrundeliegende Publikation:"
                },
            element html5:ul
            {
                for $publikation in single:witness( $node/tei:fileDesc/tei:sourceDesc, $mode )/descendant::tei:witness/tei:bibl
                    return
                        element html5:li
                        {
                            $publikation/preceding-sibling::tei:desc/text() || " ",
                            single:linkicon($publikation, "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book")
                        }
            }
        }

    case element(tei:note) return
        if ($node/@type = "KMG") then
            (if ($node/ancestor::tei:p) then
              <br />
            else
              (),
            element xhtml:div
            {
                attribute class {"note kmg-note"},
                single:transform($node/node(), $mode, $file)
            })
        else if ($node/@type = "editorial") then
            (if ($node/ancestor::tei:p) then
              <br />
            else
              (),
            element xhtml:div
            {
                attribute class {"note kmg-note"},
                single:transform($node/node(), $mode, $file)
            })
        else
            (if ($node/ancestor::tei:p) then
              <br />
            else
              (),
            element xhtml:div
            {
                attribute class {"note"},
                single:transform($node/node(), $mode, $file)
            })

    (: vorerst unverändert :)
    case element(tei:ptr) return
      ()

    (: Auskommentiert, da tei:titlePage in der KMG nicht existiert
    case element(tei:titlePage) return
      element xhtml:span
      {
        attribute class {"title-page"},
        single:transform($node/node(), $mode, $file)
      }
    :)

    (: Auskommentiert, da tei:ab in der KMG nicht existiert
    case element(tei:ab) return
      if ($node[@type = "half-title"]) then
        element xhtml:div
        {
          attribute class {$node/@type},
          single:transform($node/node(), $mode, $file)
        }
      else
        element xhtml:span
        {
          attribute class {"ab"},
          single:transform($node/node(), $mode, $file)
        }
    :)

    (: vorerst unverändert :)
    case element(tei:cb) return
      element xhtml:span
      {
        attribute class {"cb"},
        string("|" || replace($node/@edRef, '[# ]+', '') || $node/@n || "|")
      }

    (: vorerst unverändert :)
    case element(tei:pb) return
      if (matches($node/@edRef, $mode)) then
        (
          commons:make-whitespace-before($node),
          element xhtml:span
          {
            attribute class {"pb"},
            concat
            (
              string("|" || replace($node/@edRef, '[# ]+', '')),
              (if ($node/@type = 'sp') then string("[") else ""),
              string($node/@n),
              (if ($node/@type = 'sp') then string("]") else ""),
              string("|")
            )
          }
        )
      else if (not($node/following-sibling::*[1][self::tei:pb])) then
        commons:make-whitespace-after($node)
      else
        ()
        
    case element(tei:milestone) return
        if ($node/@unit = "page"
        and matches($node/@edRef, $mode)) then
            (
              commons:make-whitespace-before($node),
              element xhtml:span
              {
                attribute class {"pb"},
                concat
                (
                  string("|" || replace($node/@edRef, '[# ]+', '')),
                  (if ($node/@type = 'sp') then string("[") else ""),
                  string($node/@n),
                  (if ($node/@type = 'sp') then string("]") else ""),
                  string("|")
                )
              }
            )
        else
            ()
        
              
    (:
    : Frühere Funktion für tei:milestone
    : let $edition := replace(replace($node/@edRef, "#", ""), " ", "")
    :    return
    :      if ($node/@unit = "fn-break"
    :      and $edition = $mode) then
    :        element xhtml:span
    :        {
    :          attribute class {"fn-break"},
    :          string("|" || $edition || $node/@n || "|")
    :        }
    :      else
    :          ()
    :)


    (: vorerst unverändert :)
    case element(tei:back) return
      single:transform($node/node(), $mode, $file)

    (: angepasst an kmg :)
    case element(tei:item) return
      if ($node/parent::tei:figure) then
            ()
      else if ($node/preceding-sibling::node()[not(self::text())][position() = 1]/local-name() = 'label') then
          element xhtml:li
          {
            if ($node/@xml:id) then
              attribute id {$node/@xml:id}
            else
              (),
            attribute class {"labeled_item"},
            <xhtml:span class="list-label">{data($node/preceding-sibling::tei:label[@type="list"][1])}</xhtml:span>,
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
              single:transform($node/node(), $mode, $file)
          }
      else if ($node/preceding-sibling::node()[not(self::text())][position() = 1]/local-name() = 'app') then
          element xhtml:li
          {
            if ($node/@xml:id) then
              attribute id {$node/@xml:id}
            else
              (),
            attribute class {"labeled_item"},
            <xhtml:span class="list-label">{data($node/preceding-sibling::tei:app[1]/*[@wit="#" || $mode]/tei:label[@type="list"])}</xhtml:span>,
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
              single:transform($node/node(), $mode, $file)
          }
      else
          element xhtml:li
          {
            if ($node/@xml:id) then
              attribute id {$node/@xml:id}
            else
              (),
            attribute class {"unlabeled_item"},
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
              single:transform($node/node(), $mode, $file)
          }
          


    (: Unnötige Spezifizierungen auskommentiert, Attributswerte angepasst :)
    case element(tei:choice) return
        if ($node/tei:corr[@type = "editorial"]) then
            element html5:span
            {
                attribute class {"resp-tt"},
                if ($node/@copyOf) then
                  single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
                else
                  single:transform($node/node(), $mode, $file),
                element html5:span
                {
                    attribute class {"resp-tttext"},
                    "Verantwortlich: " || data(single:linkelement( $node/tei:corr )) || " ",
                    if (single:fullink( $node/tei:corr )) then
                        single:fulllinkicon( $node/tei:corr, "gnd:", "http://d-nb.info/gnd/", "", "fas fa-user" )
                    else
                        ()
                }
            }
        else
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
              single:transform($node/node(), $mode, $file)
        (: if ($node//tei:abbr) then
            single:transform($node//tei:abbr, $mode, $file)
        else if ($node//tei:orig) then
        single:transform($node//tei:orig, $mode, $file)
        (: compositor's errors :)
          else if ($node/tei:corr
          and ($node/..[@type = "typo-correction"][contains(@wit, $mode)]
            or not($node/parent::tei:lem/following-sibling::tei:rdg[contains(@wit, $mode)]))) then
            (single:transform($node/tei:corr, $mode, $file),
            single:make-typo-corr($node, $mode, $file),
            (: compositor's error with authorial correction :)
            if ($node/tei:corr[@type = "authorial"]) then
              element xhtml:a
              {
                attribute href {commons:get-link-url($node, $mode)},
                <i class="fas fa-pen-fancy"></i>
              }
            else
              ()
            )
          (: authorial corrigenda :)
          else if ($node/tei:corr[@type = "errata"]) then
            (single:transform($node//tei:corr, $mode, $file),
            element xhtml:a
            {
              attribute href {commons:get-link-url($node, $mode)},
              string("@TEST@"),
              <i class="fas fa-pen-fancy"></i>
            })
          else
            () (:TODO: Satzfehler :)
            :)
    
    case element(tei:sic) return
        element html5:span
        {
            attribute class {"kmg linethrough"},
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        }
    
    case element(tei:corr) return
        element html5:span
        {
            attribute class {"kmg"},
            text{" "},
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        }

    (: Es kann noch andere @result außer p und item geben, insofern muss dies angepasst werden :)
    case element(tei:join) return
      switch($node/@result)
      case "p" return
        element xhtml:div
        {
          attribute class {"p"},
          single:apply-join($node, $mode, $file)
        }
      case "item" return
        element xhtml:li
        {
          attribute class {"list-item"},
          single:apply-join($node, $mode, $file)
        }
      (: the default case is that join has no @result :)
      default return
        single:apply-join($node, $mode, $file)


    (: Frage: bei xhtml:div bleiben oder zu xhtml:p ändern? Dann noch die drei @rend von tei:p einpflegen :)
    case element(tei:p) return
      (: since we have xhtml:div nested in p, e.g. for compositor's
      errors, we need to choose xhtml:div here instead of xhtml:p.
      otherwise the HTML content model is violated. :)
      if ($node/ancestor::tei:figure) then
            element html5:figcaption
            {
               single:transform($node/node(), $mode, $file) 
            }
      else
          element xhtml:div
          {
                if ($node/@rend= "blockquote") then
                    attribute class {"blockquote p"}
                else if ($node/@rend= "blockquoteitalics") then
                    attribute class {"blockquote italics p"}
                else
                    attribute class {"p"},
            (:if (preceding-sibling::p
            or (ancestor::app[@type = 'structural-variance']/parent::div[@type = 'section']
            and ancestor::div[@type = 'section']/child::*[2] != ancestor::app[@type = 'structural-variance']):)
            (: in case a paragraph starts directly with an app[@type = 'structural-variance'] :)
            (:or ancestor::app[@type = 'structural-variance']/parent::div[@type = 'section']
              and not(//*[@xml:id = $node/@copyID]/ancestor::app[@type = 'structural-variance'])
            or ancestor::app/preceding-sibling::*[1][self::p]
            or preceding-sibling::*[1][self::list]
            or preceding-sibling::*[1]/child::*[last()][self::list]
            or preceding-sibling::*[1][self::app]/lem/child::*[last()][self::list])
              then
              attribute{"style"}{"text-indent: 15px;"}
              else
              attribute{"style"}{"margin-left: 0;"},:)
            if ($node/@copyOf) then
              single:apply-corresp-node($node, $node/@copyOf, $mode, $file, false())
              else
              single:transform($node/node(), $mode, $file)
            }

    case element(tei:ref) return
        element html5:span {
            attribute class { "ref" },
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file),
            string(" "),
            if (starts-with($node/@target, "#" )) then
                element html5:a {
                    attribute class {"text-link"},
                    attribute href {$node/@target},
                    element html5:i {
                        attribute class { "fas fa-link" }
                    }
                }
            else
                element html5:a {
                    attribute class {"text-link"},
                    attribute href {replace($node/@target, "textgrid:", "/exist/apps/sade/textgrid/data/kmg/")},
                    element html5:i {
                        attribute class { "fas fa-link" }
                    }
                },
            if (commons:starts-next-text-with-punctuation($node, $mode)) then
                ()
            else
                text{" "}
        }

    (: vorerst unverändert :)
    case element(tei:lb) return
      if ($node/preceding-sibling::*[self::tei:titlePart or self::tei:byline
      or self::tei:docImprint or self::tei:head]) then
        ()
      else
        <br />

    (: Auskommentiert, da tei:docImprint in der KMG nicht existiert
    case element(tei:docImprint) return
        (single:transform($node/node(), $mode, $file),
        <br />)
    :)

    (: Auskommentiert, da tei:titlePage in der KMG nicht existiert
    case element(tei:supplied) return
      (
        text {"["},
        single:transform($node/node(), $mode, $file),
        text{"]"}
      )
    :)

    (: vorerst unverändert :)
    case element (tei:floatingText) return
        element xhtml:div
        {
            attribute class {"floating-text"},
            single:transform($node/node(), $mode, $file)
        }


    (: vorerst unverändert :)
    case element (tei:signed) return
        element xhtml:div
        {
            attribute class {"signed"},
            single:transform($node/node(), $mode, $file)
        }

    case element(tei:quote) return
        if ($node/descendant::tei:lg) then
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        else if ($node/@rend = "none") then
            single:qclass( $node, $mode, $file, "quote" )
        else if ($node[contains(@rend, "double")]) then
            (if ($node[contains(@rend, "qm_low")]) then
                single:qclass( $node, $mode, $file, "quote double-qm-low-l double-qm-low-r" )
            else if ($node[contains(@rend, "qm_high")]) then
                single:qclass( $node, $mode, $file, "quote double-qm-high-l double-qm-high-r" )
            else if ($node[contains(@rend, "g_out")]) then
                single:qclass( $node, $mode, $file, "quote double-q-out-l double-q-out-r" )
            else if ($node[contains(@rend, "g_in")]) then
                single:qclass( $node, $mode, $file, "quote double-q-in-l double-q-in-r" )
            else
                (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                    single:qclass( $node, $mode, $file, "quote double-qm-low-l double-qm-low-r" )
                else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                    single:qclass( $node, $mode, $file, "quote double-qm-high-l double-qm-high-r" )
                else if (contains(single:wit-rend($node, $mode), "g_out")) then
                    single:qclass( $node, $mode, $file, "quote double-g-out-l double-g-out-r" )
                else if (contains(single:wit-rend($node, $mode), "g_in")) then
                    single:qclass( $node, $mode, $file, "quote double-g-in-l double-g-in-r" )
                else
                    single:qclass( $node, $mode, $file, "quote double-l double-r" )
                )
            )
        else if ($node[contains(@rend, "single")]) then
            (if ($node[contains(@rend, "qm_low")]) then
                single:qclass( $node, $mode, $file, "quote single-qm-low-l single-qm-low-r" )
            else if ($node[contains(@rend, "qm_high")]) then
                single:qclass( $node, $mode, $file, "quote single-qm-high-l single-qm-high-r" )
            else if ($node[contains(@rend, "g_out")]) then
                single:qclass( $node, $mode, $file, "quote single-q-out-l single-q-out-r" )
            else if ($node[contains(@rend, "g_in")]) then
                single:qclass( $node, $mode, $file, "quote single-q-in-l single-q-in-r" )
            else
                (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                    single:qclass( $node, $mode, $file, "quote single-qm-low-l single-qm-low-r" )
                else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                    single:qclass( $node, $mode, $file, "quote single-qm-high-l single-qm-high-r" )
                else if (contains(single:wit-rend($node, $mode), "g_out")) then
                    single:qclass( $node, $mode, $file, "quote single-g-out-l single-g-out-r" )
                else if (contains(single:wit-rend($node, $mode), "g_in")) then
                    single:qclass( $node, $mode, $file, "quote single-g-in-l single-g-in-r" )
                else
                    single:qclass( $node, $mode, $file, "quote single-l single-r" )
                )
            )
        else
            ()

    case element(tei:q) return
        if ($node/descendant::tei:lg) then
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        else if ($node[contains(@rend, "double")]) then
            (if ($node[contains(@rend, "qm_low")]) then
                single:qclass( $node, $mode, $file, "double-qm-low-l double-qm-low-r" )
            else if ($node[contains(@rend, "qm_high")]) then
                single:qclass( $node, $mode, $file, "double-qm-high-l double-qm-high-r" )
            else if ($node[contains(@rend, "g_out")]) then
                single:qclass( $node, $mode, $file, "double-q-out-l double-q-out-r" )
            else if ($node[contains(@rend, "g_in")]) then
                single:qclass( $node, $mode, $file, "double-q-in-l double-q-in-r" )
            else
                (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                    single:qclass( $node, $mode, $file, "double-qm-low-l double-qm-low-r" )
                else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                    single:qclass( $node, $mode, $file, "double-qm-high-l double-qm-high-r" )
                else if (contains(single:wit-rend($node, $mode), "g_out")) then
                    single:qclass( $node, $mode, $file, "double-g-out-l double-g-out-r" )
                else if (contains(single:wit-rend($node, $mode), "g_in")) then
                    single:qclass( $node, $mode, $file, "double-g-in-l double-g-in-r" )
                else
                    single:qclass( $node, $mode, $file, "double-l double-r" )
                )
            )
        else if ($node[contains(@rend, "single")]) then
            (if ($node[contains(@rend, "qm_low")]) then
                single:qclass( $node, $mode, $file, "single-qm-low-l single-qm-low-r" )
            else if ($node[contains(@rend, "qm_high")]) then
                single:qclass( $node, $mode, $file, "single-qm-high-l single-qm-high-r" )
            else if ($node[contains(@rend, "g_out")]) then
                single:qclass( $node, $mode, $file, "single-q-out-l single-q-out-r" )
            else if ($node[contains(@rend, "g_in")]) then
                single:qclass( $node, $mode, $file, "single-q-in-l single-q-in-r" )
            else
                (if (contains(single:wit-rend($node, $mode), "qm_low")) then
                    single:qclass( $node, $mode, $file, "single-qm-low-l single-qm-low-r" )
                else if (contains(single:wit-rend($node, $mode), "qm_high")) then
                    single:qclass( $node, $mode, $file, "single-qm-high-l single-qm-high-r" )
                else if (contains(single:wit-rend($node, $mode), "g_out")) then
                    single:qclass( $node, $mode, $file, "single-g-out-l single-g-out-r" )
                else if (contains(single:wit-rend($node, $mode), "g_in")) then
                    single:qclass( $node, $mode, $file, "single-g-in-l single-g-in-r" )
                else
                    single:qclass( $node, $mode, $file, "single-l single-r" )
                )
            )
        else
            ()
    
    case element(tei:figure) return
        element html5:figure
        {
            attribute class {"image"},
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        }
    
    case element(tei:graphic) return
        element html5:a
        {
            attribute href {replace($node/[@url], "textgrid:", "/exist/apps/sade/textgrid/data/kmg-images/") || ".jpg"},
            element html5:img
            {
                attribute src {replace($node/[@url], "textgrid:", "/exist/apps/sade/textgrid/data/kmg-images/") || ".jpg"},
                attribute alt {data(replace(string($node/ancestor::tei:figure/descendant::tei:figDesc), "@", " "))},
                if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
            }
        }
    
    case element(tei:figDesc) return
        ()

(:~
 : This transformation uses the function single:spanicon to create Link-Icons for the different tei:name-elements. If the tei:name refers to another tei:name-element in the same document, the internal link will be replaced by the external link of the other tei:element. If the tei-element does not have a ref-attribute, no link will created.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)

    case element(tei:name) return
        if ($node/@type = "art") then
            if (not($node/@ref)) then
                single:spanclass( $node, $mode, $file, "artname" )
            else
                single:spanicon($node, $mode, $file, "artname", "textgrid:", "/exist/apps/sade/textgrid/data/kmg-lido/", ".xml", "far fa-image")
        else if ($node/@type = "person") then
            if (not($node/@ref)) then
                single:spanclass( $node, $mode, $file, "persname" )
            else
                single:spanicon($node, $mode, $file, "persname", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-user")
        else if ($node/@type = "org") then
            if (not($node/@ref)) then
                single:spanclass( $node, $mode, $file, "orgname" )
            else
                single:spanicon($node, $mode, $file, "orgname", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-external-link-alt")
        else if ($node/@type = "music") then
            if (not($node/@ref)) then
                single:spanclass( $node, $mode, $file, "musicname" )
            else
                single:spanicon($node, $mode, $file, "musicname", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-music")
        else if ($node/@type = "periodica") then
            if (not($node/@ref)) then
                single:spanclass( $node, $mode, $file, "periodname" )
            else
                single:spanicon($node, $mode, $file, "periodname", "gnd:", "http://d-nb.info/gnd/", "", "fas fa-newspaper")
        else if ($node/@type = "brand") then
            if (not($node/@ref)) then
                single:spanclass( $node, $mode, $file, "brandname" )
            else
                single:spanicon($node, $mode, $file, "brandname", "gnd:", "http://d-nb.info/gnd/", "", "far fa-copyright")
        else
            (element html5:span
            {
              attribute class {"ref"},
              if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file),
              string(" "),
              element html5:a
              {
                attribute class {"text-link"},
                attribute href {$node/[@ref]},
                <i class="fas fa-external-link-alt"></i>
              }
            },
            if (commons:starts-next-text-with-punctuation($node, $mode)) then
              ()
            else
              text{" "})
    
(:~
 : This transformation uses the function single:spanicon to create Link-Icons for tei:bibl with different classes for @type="KMG". It creates a whitespace if the tei:bibl follows directly a tei:quote or tei:q.
 :
 : @author Max Zeterberg
 : @version 1.0
 :)
    
    case element(tei:bibl) return
        if ($node/@type = "KMG") then
            if ($node/preceding-sibling::tei:quote or $node/preceding-sibling::tei:q) then
                (element html5:span
                {
                    " "
                },
                single:spanicon($node, $mode, $file, "bibl KMG", "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book")
                )
            else
                single:spanicon($node, $mode, $file, "bibl KMG", "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book")
        else
            if ($node/preceding-sibling::tei:quote or $node/preceding-sibling::tei:q) then
                (element html5:span
                {
                    " "
                },
                single:spanicon($node, $mode, $file, "bibl", "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book")
                )
            else
                single:spanicon($node, $mode, $file, "bibl", "zotero:", "http://zotero.org/groups/2206325/items/", "", "fas fa-book")


    case element(tei:table) return
        if ($node/tei:trailer) then
            element html5:figure
            {
               attribute class {"tablefig"},
               element html5:table
               {
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file) 
               },
               element html5:figcaption
               {
                   if ($node/tei:trailer/@copyOf) then
                        single:apply-corresp-node($node/tei:trailer, $node/tei:trailer/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/tei:trailer/node(), $mode, $file) 
               }
            }
        else
            element html5:figure
            {
               attribute class {"tablefig"},
               element html5:table
               {
                    if ($node/@copyOf) then
                        single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                    else
                        single:transform($node/node(), $mode, $file) 
               }
            }

    case element(tei:row) return
        element html5:tr
        {
            if ($node/@copyOf) then
                single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
            else
                single:transform($node/node(), $mode, $file)
        }

    case element(tei:cell) return
        if ($node/parent::tei:row/parent::tei:table/@rend = "no_border") then
            element html5:td
            {
               attribute class {"no_border"},
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
        else if ($node/parent::tei:row/@rend = "no_border") then
            element html5:td
            {
               attribute class {"no_border"},
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
        else if ($node/@rend) then
            element html5:td
            {
               attribute class {data($node/@rend)},
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
        else
            element html5:td
            {
               if ($node/@cols) then
                   attribute colspan {data($node/@cols)}
               else
                   (),
               if ($node/@rows) then
                   attribute rowspan {data($node/@rows)}
               else
                   (),
               if ($node/@copyOf) then
                    single:apply-corresp-node($node, $node/@copy-of, $mode, $file, false())
                else
                    single:transform($node/node(), $mode, $file)
               
            }
    
    case element(tei:trailer) return
        ()

    (: vorerst unverändert :)
    default return
      single:transform($node/node(), $mode, $file),
      commons:make-whitespace-after($node)
  )
};


(:~
 : Finds and applies the node that is referenced with @copyOf.
 : In case of tei:join we have to apply only the child nodes of the corresponding
 : nodes because tei:join defines a new resulting element in @result that makes
 : the original nodes obsolete.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The current node
 : @param $copyOf A reference to another node
 : @param $mode The current edition
 : @param $file
 : @param $join A flag indicating if the calling element is a tei:join
 : @return The transformed node the @copyOf refers to
 :)
declare function single:apply-corresp-node($node as node()*, $copyOf as xs:string, $mode as
xs:string, $file as xs:string, $apply-children as xs:boolean)
as node()*
{
  let $copyOf := replace($copyOf, "#", "")
  let $corresp-node := single:find-corresp-node($node, $copyOf)
  return
      if ($apply-children) then
          for $child in $corresp-node return
              single:transform($child, $mode, $file)
      else
            single:transform($corresp-node, $mode, $file)
};


(:~
 : Retrieves the target node of a given @copyOf-reference. 
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The current node
 : @param $copyOf A reference to another node
 : @return The node the @copyOf refers to
 :)
declare function single:find-corresp-node($node as node()*, $copyOf as xs:string) as node()?{
    $node/root()//*[@xml:id = $copyOf]
};


(:~
 : Creates the icon for a compositor's error and initiates the construction of
 : the respective modal div.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The the current tei:choice
 : @param $mode The current edition
 : @return The modal div, a whitespace and the icon
 :)
declare function single:make-typo-corr($node as element(tei:choice),
$mode as xs:string, $file as xs:string) as node()*
{
  let $random-id := single:make-random-id()
  let $id := util:hash($random-id, "md5")
  let $edition :=
    if ($node/parent::tei:lem[not(@wit)]) then
      (: single file names don't have a hyphen, only the ones that are the
      result of semantic splitting :)
      if (matches($file, "-")) then
        let $author := substring-before($file, "-")
        let $full-file := doc($config:full-xmls || $author || "-full.xml")
        return $full-file//tei:witness[@n = "base-text"]/@xml:id/string()
      else
        $node/ancestor::tei:TEI//tei:witness[@n = "base-text"]/@xml:id
    else
      $mode
  return
    (single:make-typo-corr-modal($node, $edition, $mode, $id, $file),
    text{" "},
    <a class="comp-error" data-target="#{$id}">
      <i class="fa fa-wrench" aria-hidden="true" data-toggle="modal" />
    </a>)
};


(:~
 : Creates the modal div for a compositor's error.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $node The the current tei:choice
 : @param $edition The edition in which an error occurs
 : @param $mode The current edition
 : @param $id The target id
 : @return The modal div
 :)
declare function single:make-typo-corr-modal($node as element(tei:choice),
$edition as xs:string, $mode as xs:string, $id as xs:string, $file as xs:string)
as element(xhtml:div)
{
  <div class="modal fade" role="dialog" id="{$id}">
    <div class="modal-dialog modal-typo">
      <div class="modal-content">
        <div class="modal-header">
          <button type="button" class="close" data-dismiss="modal">
            × <!-- times -->
          </button>
          {
            if ($node/tei:corr/@type = "authorial") then
              let $sic :=
                if ($node/tei:sic[not(child::node() or child::node())]) then
                  <em>&#248;</em>
                else
                  single:transform($node/tei:sic, $mode, $file)
              return
                (string("Autorschaftliche Korrektur von: "),
                $sic,
                string(" (" || $edition || ")"))
            else
              let $sic :=
                if ($node/tei:sic[not(child::* or child::node())]) then
                  <em>&#248;</em>
                else
                  single:transform($node/tei:sic, $mode, $file)
              return
                (string("Editorische Korrektur von: "),
                $sic,
                string(" (" || $edition || ")"))
          }
        </div>
      </div>
    </div>
  </div>
};


declare function single:is-next-text-in-app($node as node()*) as xs:boolean
{
  if ($node/following::text()[1][normalize-space(.) != ""][ancestor::tei:app])then
    true()
  else
    false()
};

declare function single:is-next-app-omitted($node as node()*, $mode as xs:string)
as xs:boolean
{
  let $next-app := $node/following-sibling::*[1][self::tei:app]
  return
    if ($next-app/child::tei:rdg[matches(@wit, $mode) and @type = "om"]) then
      true()
    else
      false()
};

(:~
 : In cases like tei:join several references are stated in a @target. This
 : function returns all corresponding nodes.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $target The @target of a e.g. tei:join
 : @param $mode The current edition
 : @return The nodes the @target refers to
 :)
declare function single:apply-join($node as element(tei:join), $mode as xs:string,
$file as xs:string) as node()*
{
    let $apply-only-children :=
        if ($node/@scope = "root") then
            false()
        else
            true()
    return
        for $copyOf in tokenize($node/@target, " ")
            let $applied-node := single:apply-corresp-node($node, $copyOf, $mode, $file, $apply-only-children)
            let $first-text := $applied-node[1]/descendant-or-self::text()[1]
            let $conjunction :=
              if (starts-with($first-text, ",")) then
                text{""}
              else
                text{" "}
            return ($conjunction, $applied-node)
};

(:~
 : Creates a random ID of 40 letters consisting of Basic Latin letters and
 : numbers. This function is needed since the usage of generate-id() (which we
 : originally used for the linking between icons and a modal dialogue) lead to
 : ID duplettes.
 :)
declare function single:make-random-id() as xs:string
{
    let $charset := "ABCDEFGHIJKLMNOPQRSTUVXYZabcdefghijklmnopqrstuvwxyz0123456789"
    let $id :=
        for $iii in 1 to 40 return
            let $number := util:random(40)
            return
                substring($charset, $number, 1)
    return string-join($id, "")
};
