xquery version "3.1";

(: 
 : This XQuery looks for a file called zoteroconfig.xml in the database
 : home directory under $EXIST_HOME/etc.
 : If there is one, then this script will copy the file 
 : into /db/apps/zoteroapi/config.xml to configure the 
 : Zotero API module.
 : 
 : This XQuery is copied via post-install.xq into 
 : /db/system/autostart
 : and if 
 :  <trigger class="org.exist.collections.triggers.XQueryStartupTrigger"/>
 : is enable in conf.xml
 : this script gets executed at each existdb startup.
 :  
:)

let $log := util:log-system-out("[zoteroapi] Autostart: config-zotero.xq")

let $fileSeparator := util:system-property("file.separator")

let $configFilePath := system:get-exist-home() || $fileSeparator || "etc" || $fileSeparator || "zoteroconfig.xml"

let $configFile := if (file:exists($configFilePath)) then
    file:read($configFilePath)
    else ()

return 
    if (not ($configFile)) then
        util:log-system-out("[zoteroapi] skipped, zoteroconf.xml not found.")
    else 
        (
        util:log-system-out("[zoteroapi] found zoteroconfig.xml"),
        xmldb:remove("/db/apps/zoteroapi", "config.xml"),
        xmldb:store("/db/apps/zoteroapi", "config.xml", $configFile),
        util:log-system-out("[zoteroapi] updated zoteroapi/config.xml.")
        )


